@@ -1,16 +1,18 @@
 import dataclasses
 import datetime
 import functools
-import inspect
 import logging
+import types
+import typing
 from dataclasses import dataclass, field
 from decimal import Decimal
+from enum import Enum
 from enum import StrEnum
 from types import UnionType
-from typing import Optional, Type, Any, Union, ClassVar
+from typing import ClassVar
+from typing import Optional, Type, Any, Union
 
 from apispec import APISpec
-from dict_caster.extras import first
 from http_tools import Answer
 from init_helpers.dict_to_dataclass import NoValue
 
@@ -145,7 +147,7 @@ class OpenApiWrapper:
         spec = APISpec(
             title=f"{self.context.company}:{self.context.project_group}:{self.context.project_name}",
             version=self.version,
-            openapi_version="3.0.2"
+            openapi_version="3.1.0"
         )
         for endpoint in self.endpoints:
             self._add_endpoint_to_spec(spec, endpoint)
@@ -194,28 +196,26 @@ class OpenApiWrapper:
         return result
 
     @functools.cache
-    def _add_schema_to_spec(self, spec: APISpec, schema: type | UnionType, default: Any = NoValue) -> Union[str, dict]:
+    def _add_schema_to_spec(self, spec: APISpec, schema: type | UnionType, default: Any = NoValue) -> dict:
         logger.debug('_add_schema_to_spec: %s', schema)
-        optional = False
 
-        if getattr(schema, '__origin__', None) is Union:
-            type_args = getattr(schema, '__args__', None)
-            non_none_types = [type_ for type_ in type_args if type_ is not type(None)]
-            if len(non_none_types) != 1:
-                raise TypeError(f"Got unsupported type: {schema}")
-            schema = first(non_none_types)
-            optional = True
-
-        origin_type = getattr(schema, '__origin__', None)
-        if origin_type is list:
+        origin_type = typing.get_origin(schema)
+        type_args = typing.get_args(schema)
+        if origin_type in (typing.Union, types.UnionType):
+            description = {
+                "anyOf": [self._add_schema_to_spec(spec, arg) for arg in type_args],
+            }
+        elif origin_type is list:
+            assert type_args and len(type_args) == 1, f'bad list element type: {type_args}'
             description = {
                 "type": "array",
-                "items": self._add_schema_to_spec(spec, getattr(schema, '__args__')[0])
+                "items": self._add_schema_to_spec(spec, type_args[0])
             }
         elif origin_type is dict:
+            assert type_args and len(type_args) == 2, f'bad dict item types: {type_args}'
             description = {
                 "type": "object",
-                "additionalProperties": self._add_schema_to_spec(spec, getattr(schema, '__args__')[1])
+                "additionalProperties": self._add_schema_to_spec(spec, type_args[1])
             }
         elif origin_type is not None:
             raise TypeError(f"Unknown origin_type: {schema}")
@@ -233,18 +233,18 @@ class OpenApiWrapper:
             name = f"schema_{schema.__name__}"
             logger.debug('schema: %s, %s, %s', name, schema, list(description["properties"]))
             spec.components.schema(component_id=name, component=description)
-            return name
+            return spec.components.get_ref("schema", name)
         elif schema is bool:  # MUST be before int
             description = {"type": "boolean"}
         elif issubclass(schema, int):
             description = {
                 "type": "integer",
-                # "format": "int32", "default": 0
+                # "format": "int32"
             }
         elif issubclass(schema, float):
             description = {
                 "type": "number",
-                # "format": "int32", "default": 0
+                # "format": "int32"
             }
         elif issubclass(schema, bytes):
             description = {
@@ -255,14 +255,18 @@ class OpenApiWrapper:
             description = {"type": "string"}
         elif issubclass(schema, Decimal):
             description = {"type": "string", 'pattern': r'^\d*\.?\d*$'}
+        elif schema in (None, type(None)):
+            return {"type": "null"}
         else:
             raise TypeError(f"Unknown type: {schema}")
 
-        if optional:
-            description['nullable'] = optional
+        if isinstance(schema, type) and issubclass(schema, Enum):
+            description['enum'] = [e.value for e in schema]
 
         if default is None or isinstance(default, (int, float, str, list, dict)):
             description['default'] = default
+        # if default == NoValue:  # TODO: think about it, should we place "required" inside schema properties or not?
+        #     description['required'] = True
 
         return description
 
